/**
 *******************************************************************************
 * @file    can.c
 * @brief   This file provides API functions for A-CAN.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "can.h"
#include "bsp.h"

#if defined(__CAN_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/** 
 *  @defgroup Null Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
#define REG_CAN_NULL                  ((void *)0)
/**
 *  @}
 */ /* End of group Null */

/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define REG_CAN_PARAM_OK              ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define REG_CAN_PARAM_NG              ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of name Parameter CheckParamResult */


/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group REG_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/**
 *  @}
 */ /* End of group REG_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  CAN_BitConfigure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void CAN_BitConfig(TSB_CAN_TypeDef *p_can)
{  
  /* Can SW Reset [CANMCR]<SRES> to '1'       */
  p_can->MCR = 0x00000081;
  
  /* wait for over 16 fsys Clock*/
  wait(3);
  /* Setting Prescaler [CANBCR1]<BRP[9:0]> to '1'  */
  p_can->BCR1 = 0x00000001;
  /* Setting Prescaler [CANBCR2]<SJW[1:0]> to '11:4*TQ',<SAM> to '0',<TSEG2[2:0]> to '111:8*TQ', <TSEG1[3:0]> to '1110:15*TQ'  */
  p_can->BCR2 = 0x0000037E;
  /* Test loop back mode, set :CANMCR<TSTLB> to '1' */  
  p_can->MCR = 0x00000280;
  /* Setting Normal operation mode, set :CANMCR<CCR> to '0' */    
  p_can->MCR = 0x00000200;
  /* Checking Normal operation mode, set :CANGSR<CCE> to '0' */
  while((p_can->GSR & 0x00000080) != 0)
  {
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  CAN_RX_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void CAN_RX_Config(TSB_CAN_TypeDef *p_can,TSB_CANBMB_TypeDef *p_canmd)
{
  uint32_t tmp = 0;
  /* Configure mailbox No.10 for reception: set [CANMC]<MDn> to '1'*/
  p_can->MD = MSG_BOX_RX_BITNO;
  /* Setup mailbox No.10 identifier and [CANMBnID]<IDE> bit for Standard ID or extended ID
      set <GAME_LAME> to '1', <GAME_LAME> to '1' */
  p_canmd->ID = (MSG_ID | 0x60000000);
  /* Setup mailbox No.10 set [CANMBnTSVMCF]<RTR> to '0' and <DLC[3:0]> to '1000' */
  p_canmd->TSVMCF = 0x00000008;
  /* Enable mailbox No.10 : set [CANMC]<MCn> to '1' */
  tmp = 0;
  tmp = p_can->MC;
  p_can->MC = (tmp | MSG_BOX_RX_BITNO);
  p_canmd->DL = 0x0U; /* Reset data mailbox10 */
  p_canmd->DH = 0x0U; /* Reset data mailbox10 */
}


/*--------------------------------------------------*/
/** 
  * @brief  CAN_TX_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void CAN_TX_Config(TSB_CAN_TypeDef *p_can,TSB_CANBMB_TypeDef *p_canmd)
{
  uint32_t tmp = 0;

  /*Disable mailbox: set [CANMC]<MCn> to '0'*/
  p_can->MC = 0x0;
  /* Configure mailbox No.0 for reception: set [CANMC]<MDn> to '0'*/
  tmp = p_can->MD;
  p_can->MD = (tmp & ~MSG_BOX_TX_BITNO);
  /* Setup mailbox No.0 identifier and [CANMBnID]<IDE> bit for Standard ID or extended ID
      set <RFH> to '1'                                            */
  p_canmd->ID = (MSG_ID | 0x20000000);
  /* Setup mailbox No.0 set [CANMBnTSVMCF]<RTR> to '0' and <DLC[3:0]> to '1000' */
  p_canmd->TSVMCF = 0x00000008;
  /* Enable mailbox No.0 : set [CANMC]<MCn> to '1'                */
  tmp = 0;
  tmp = p_can->MC;
  p_can->MC = (tmp | MSG_BOX_TX_BITNO);
}
/**
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__CAN_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */


