/** 
 *******************************************************************************
 * @file    can.h
 * @brief   This file provides all the functions prototypes for A-CAN.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __CAN_H
#define __CAN_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */

/** ---------------------------------------
 *  @defgroup CANCR1 CANCR1
 *  @brief    CAN Control Register 1.
 *  @{
 */

/** 
 *  @defgroup CANCR1_CANWEN CANWEN
 *  @brief    [Bit7:0]: Register Write Enable/Disable Control.
 *  @{
 */
#define REG_CAN_CANCR1_CANWEN_DISABLE        ((uint32_t)0x00000006)        /*!< Register Write Disable.     */
#define REG_CAN_CANCR1_CANWEN_ENABLE         ((uint32_t)0x000000F9)        /*!< Register Write Enable.      */
/**
 *  @}
 */ /* End of group CANCR1_CANWEN */

/**
 *  @}
 */ /* End of group CANCR1 */
/** ---------------------------------------
 *  @defgroup CANCR2 CANCR2
 *  @brief    CAN Control Register 2.
 *  @{
 */

/** 
 *  @defgroup CANCR2_CANEN CANEN
 *  @brief    [Bit7:0]: Frequency Detection Enable/Disable Control.
 *  @{
 */
#define REG_CAN_CANCR2_CANEN_DISABLE         ((uint32_t)0x00000000)        /*!< Frequency Detection Disable.  */
#define REG_CAN_CANCR2_CANEN_ENABLE          ((uint32_t)0x000000E4)        /*!< Frequency Detection Enable.   */
/**
 *  @}
 */ /* End of group CANCR2_CANEN */
/**
 *  @}
 */ /* End of group CANCR2 */
/** ---------------------------------------
 *  @defgroup CANRST CANRST
 *  @brief    CAN RESET Control Register.
 *  @{
 */

/** 
 *  @defgroup CANRST_CANRSTEN CANRSTEN
 *  @brief    [Bit0]: RESET Enable/Disable Control.
 *  @{
 */
#define REG_CAN_CANRST_CANRSTEN_DISABLE      ((uint32_t)0x00000000)        /*!< RESET Disable.              */
#define REG_CAN_CANRST_CANRSTEN_ENABLE       ((uint32_t)0x00000001)        /*!< RESET Enable.               */
/**
 *  @}
 */ /* End of group CANCR2_CAN2EN */
 /**
 *  @}
 */ /* End of group CANRST */

/** ---------------------------------------
 *  @defgroup CANSTAT CANSTAT
 *  @brief    CAN Status Register.
 *  @{
 */

/** 
 *  @defgroup CANSTAT_CANBUSY CANBUSY
 *  @brief    [Bit1]: ODD Running status.
 *  @{
 */
#define REG_CAN_CANSTAT_CANBUSY_MASK         ((uint32_t)0x00000002)        /*!< For bit MASK.                 */
#define REG_CAN_CANSTAT_CANBUSY_SUSPENDED    ((uint32_t)0x00000000)        /*!< CAN Susupended.               */
#define REG_CAN_CANSTAT_CANBUSY_RUNNING      ((uint32_t)0x00000002)        /*!< CAN Running.                  */
/**
 *  @}
 */ /* End of group CANSTAT_CANBUSY */
/** 
 *  @defgroup CANSTAT_FRQERR FRQERR
 *  @brief    [Bit0]: ODD Error Detection Flag.
 *  @{
 */
#define REG_CAN_CANSTAT_FRQERR_MASK          ((uint32_t)0x00000001)        /*!< For bit MASK.                 */
#define REG_CAN_CANSTAT_FRQERR_NOERROR       ((uint32_t)0x00000000)        /*!< Not Error occurred.           */
#define REG_CAN_CANSTAT_FRQERR_ERROR         ((uint32_t)0x00000001)        /*!< Error occurred.               */
/**
 *  @}
 */ /* End of group CANSTAT_CANBUSY */
 /**
 *  @}
 */ /* End of group CANSTAT */
/** ---------------------------------------
 *  @defgroup CANMON1 CANMON1
 *  @brief    CAN Detection Monitor Clock Setting.
 *  @{
 */

/** 
 *  @defgroup CANMON1_CANMON1 CANMON1
 *  @brief    [Bit0]: Frequency Detection Enable/Disable Control.
 *  @{
 */
#define REG_CAN_CANMON1_CANMON1_EHOSC           ((uint32_t)0x00000000)        /*!< Monitor Clock for EHOSC.      */
#define REG_CAN_CANMON1_CANMON1_FC              ((uint32_t)0x00000001)        /*!< Monitor Clock for fc.         */
/**
 *  @}
 */ /* End of group CANMON1_CANMON1 */
 /**
 *  @}
 */ /* End of group CANMON1 */
/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup MSG_ID_CFG Message ID  Configuration
 *  @brief    Message ID  Configuration.
 *  @{
 */
#define MSG_ID                      (0x00000014U)
#define BAUDRATE_PRESCALER          (0x000003FFU)
#define MSG_BOX_TX_BITNO            (0x00000001U)
#define MSG_BOX_RX_BITNO            (0x00000400U)

 /** 
 *  @defgroup  FSYS_EN FSYS Enable/Disable
 *  @brief     FSYS Enable/Disable
 *  @{
 */
#define FSYS_DISABLE                    ((uint32_t)0x00000000)    /*!< Disable                 */
#define FSYS_ENABLE                     ((uint32_t)0x00000001)    /*!< Enable                  */

/**
 *  @}
 */ /* End of group REG_Exported_typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
/* NFCKCR */
void CAN_BitConfig(TSB_CAN_TypeDef *p_can);
void CAN_RX_Config(TSB_CAN_TypeDef *p_can,TSB_CANBMB_TypeDef *p_canmd);
void CAN_TX_Config(TSB_CAN_TypeDef *p_can,TSB_CANBMB_TypeDef *p_canmd);

/** 
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __CAN_H */


