/**
 *******************************************************************************
 * @file    ethm.c
 * @brief   This file provides API functions for A-ETHM.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>   
#include "txz_mdma.h"
#include "ethm.h"
#include "bsp.h"

#if defined(__ETHM_H)
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/** 
 *  @defgroup Null Null Pointer
 *  @brief    Null Pointer.
 *  @{
 */
/**
 *  @}
 */ /* End of group Null */

/** 
 *  @defgroup CheckParamResult Parameter Check Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
/**
 *  @}
 */ /* End of name Parameter CheckParamResult */


/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group REG_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_variables
 *  @{
 */
static uint32_t txDescriptor[ETH_TX_DESCRIPTOR_NUM][4];
static uint32_t rxDescriptor[ETH_RX_DESCRIPTOR_NUM][4];

static uint8_t txBuffer[2][64] __attribute__((aligned(4)));
static uint8_t rxBuffer[2][64] __attribute__((aligned(4)));
/**
 *  @}
 */ /* End of group REG_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Private_fuctions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  ETH_TDMA_Init
  * @param  p_ethm	ether register pointer
  * @retval none
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result ETH_TDMA_Init(TSB_EM_TypeDef *p_ethm)
{
	
	/* TDMA soft reset */
	TSB_EMA_BUS_MODE_SWR = 1;
	
	/* wait until soft reset over */
	while(TSB_EMA_BUS_MODE_SWR)
	{
	}
		
	/* set Bus Mode */
	p_ethm->BUS_MODE = BUSMODE_RPBL | BUSMODE_PBL;
        
	/* set receive descriptor data address */
	txDescriptor[0][0] = TX_DES0_OWN | TX_DES0_IC | TX_DES0_LS | TX_DES0_FS | TX_DES0_CRCR | TX_DES0_TER;
	txDescriptor[0][1] = TX_DES1_TBS1;
	txDescriptor[0][2] = (uint32_t)&txBuffer[0][0];
	p_ethm->TRANSMIT_DESCRIPTOR_LIST_ADDRESS = (uint32_t)(&txDescriptor[0][0]);

	rxDescriptor[0][0] = RX_DES0_OWN;
	rxDescriptor[0][1] = RX_DES1_DIC | RX_DES1_RER | RX_DES1_RBS1;
	rxDescriptor[0][2] = (uint32_t)&rxBuffer[0][0];
	p_ethm->RECEIVE_DESCRIPTOR_LIST_ADDRESS = (uint32_t)(&rxDescriptor[0][0]);

	/* set operation mode */
	p_ethm->OPERATION_MODE = OPEMODE_DT | OPEMODE_ST | OPEMODE_FUF | OPEMODE_SR;

	return TXZ_SUCCESS;
}

static TXZ_Result ETH_TMAC_Init(TSB_EM_TypeDef *p_ethm)
{
	p_ethm->TMAC_CONFIGURATION = MACCONF_SARC | MACCONF_DM | MACCONF_TE | MACCONF_RE;
	return TXZ_SUCCESS;
}
/**
 *  @}
 */ /* End of group REG_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  ETHM_Init
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void ETHM_Init(TSB_EM_TypeDef *p_ethm)
{
	/* Clock, Port init */
	p_ethm->MAC_CTRL0 = 0U;
	p_ethm->MAC_CTRL1 = MAC_CTRL1_RXCLKEN | MAC_CTRL1_TXCLKEN;
	p_ethm->MAC_CTRL2 = MAC_CTRL2_SRST;

	/* TDMA init */
	ETH_TDMA_Init(p_ethm);
	ETH_TMAC_Init(p_ethm);
}

void ETHM_Transmit(TSB_EM_TypeDef *p_ethm, uint8_t *p_txbuf, uint16_t len)
{
	txDescriptor[0][0] = TX_DES0_OWN | TX_DES0_IC | TX_DES0_LS | TX_DES0_FS | TX_DES0_CRCR | TX_DES0_TER;
	txDescriptor[0][1] = 0x00000000U | (uint32_t)len;
	txDescriptor[0][2] = (uint32_t)p_txbuf;
	p_ethm->TRANSMIT_POLL_DEMAND = TX_POLLDEMAND_TPD;
}

void ETHM_Receive(TSB_EM_TypeDef *p_ethm, uint8_t *p_rxbuf, uint16_t len)
{
	(void)memcpy(p_rxbuf, &rxBuffer[0][0], len);

	rxDescriptor[0][0] = RX_DES0_OWN;
	rxDescriptor[0][1] = RX_DES1_DIC | RX_DES1_RER | RX_DES1_RBS1;
	rxDescriptor[0][2] = (uint32_t)&rxBuffer[0][0];
	p_ethm->RECEIVE_POLL_DEMAND = RX_POLLDEMAND_RPD;
}

/**
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#endif /* defined(__ETHM_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */


