/** 
 *******************************************************************************
 * @file    ethm.h
 * @brief   This file provides API functions for A-ETHM.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __ETHM_H
#define __ETHM_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup REG
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_macro
 *  @{
 */

/** 
 *  @}
 */ /* End of group REG_Exported_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_define
 *  @{
 */
#define ETH_TX_DESCRIPTOR_NUM	 4U
#define ETH_RX_DESCRIPTOR_NUM	 4U

#define MAC_CTRL1_RXCLKEN      (0x00000002U)        /*!<  MAC CTRL1 RX CLK */
#define MAC_CTRL1_TXCLKEN      (0x00000001U)        /*!<  MAC CTRL1 TX CLK */
  
#define MAC_CTRL2_SRST         (0x01U)        /*!<  MAC CTRL2 ETHM RST */
  
#define TX_POLLDEMAND_TPD      (0x01U)        /*!<  Transmit Poll Demand TPD */
#define RX_POLLDEMAND_RPD      (0x01U)        /*!<  Receive  Poll Demand RPD */
  
#define BUSMODE_RPBL     (0x00020000U)              /*!<  Busmode RPBL */
#define BUSMODE_PBL      (0x00000100U)              /*!<  Busmode PBL */
  
#define TX_DES0_OWN      (0x80000000U)              /*!< TDES0 OWN */
#define TX_DES0_IC       (0x40000000U)              /*!< TDES0 IC */
#define TX_DES0_LS       (0x20000000U)              /*!< TDES0 LS */
#define TX_DES0_FS       (0x10000000U)              /*!< TDES0 FS */
#define TX_DES0_CRCR     (0x01000000U)              /*!< TDES0 CRCR */
#define TX_DES0_TER      (0x00200000U)              /*!< TDES0 TER */
    
#define TX_DES1_TBS1     (0x0000002AU)              /*!< TDES1 TBS1 */
  
#define RX_DES0_OWN      (0x80000000U)              /*!< RDES0 OWN */
  
#define RX_DES1_DIC     (0x80000000U)               /*!< RDES1 DIC */
#define RX_DES1_RER     (0x00008000U)               /*!< RDES1 RER */
#define RX_DES1_RBS1    (0x00000040U)               /*!< RDES1 RBS */
  
#define OPEMODE_DT      (0x02000000U)              /*!<  Opemode DT */
#define OPEMODE_ST      (0x00002000U)              /*!<  Opemode ST */
#define OPEMODE_FUF     (0x00000040U)              /*!<  Opemode FUF */
#define OPEMODE_SR      (0x00000002U)              /*!<  Opemode SR */
  
#define MACCONF_SARC    (0x30000000U)              /*!<  MAC configuration SARC */
#define MACCONF_DM      (0x00000800U)              /*!<  MAC configuration DM */
#define MACCONF_TE      (0x00000008U)              /*!<  MAC configuration TE */
#define MACCONF_RE      (0x00000004U)              /*!<  MAC configuration RE */
   
/** 
 *  @}
 */ /* End of group REG_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Registers Stucture Type Definition                                          */
/*------------------------------------------------------------------------------*/
/** @addtogroup Ether_Exported_Structures
  * @{
  */

/** 
 *  @}
 */ /* End of group Ether_Exported_Structures */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup REG_Exported_functions
 *  @{
 */
void ETHM_Init(TSB_EM_TypeDef *p_ethm);
void ETHM_Transmit(TSB_EM_TypeDef *p_ethm, uint8_t *p_txbuf, uint16_t len);
void ETHM_Receive(TSB_EM_TypeDef *p_ethm, uint8_t *p_rxbuf, uint16_t len);

/** 
 *  @}
 */ /* End of group REG_Exported_functions */

/**
 *  @}
 */ /* End of group REG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __ETHM_H */


